<?

class VA_SQL {

	var $DBType         = "";
  var $DBHost         = "";
  var $DBPort         = "";
  var $DBDatabase     = "";
  var $DBUser         = "";
  var $DBPassword     = "";
  var $DBPersistent   = false;

	/* dates formats */
	var $DatetimeMask   = array("YYYY","-","MM","-","DD"," ","HH",":","mm",":","ss");
	var $DateMask       = array("YYYY","-","MM","-","DD"," ","HH",":","mm",":","ss");
	var $TimeMask       = array("HH",":","mm",":","ss");
	var $TimestampMask  = array("YYYY","MM","DD","HH","mm","ss");

  var $UseODBCCursor  =  0;
  var $Uppercase      =  false;
  var $AutoFree       =  0;     
  var $LinkID         =  0;
  var $QueryID        =  0;
  var $PageNumber     =  0;
  var $RecordsPerPage =  0;
  var $RecordsShown   = -1;
  var $Record         =  array();
  var $Row            =  0;

  var $Errno       = 0;
  var $Error       = "";
  var $HaltOnError = "yes"; // "yes", "no", "report"

  /* public: constructor */
  function VA_SQL($query = "") {
		$this->RecordsPerPage = 0;
		$this->query($query);
  }

  function connect() 
	{
    if (!$this->LinkID) {
      if ($this->DBPersistent)
        $this->LinkID = @odbc_pconnect($this->DBDatabase, $this->DBUser, $this->DBPassword, $this->UseODBCCursor);
      else
        $this->LinkID = @odbc_connect($this->DBDatabase, $this->DBUser, $this->DBPassword, $this->UseODBCCursor);

      if (!$this->LinkID) {
				$this->Errno = @odbc_error(); 
				$this->Error = @odbc_errormsg();
        $this->halt("Connect failed.");
        return 0;
      }
    }
		return $this->LinkID;
  }
  
  function query($Query_String) {

    if ($Query_String == "") {
      return 0;
		}

		$this->RecordsShown = -1;
    $this->connect();
    
    $this->QueryID = @odbc_exec($this->LinkID, $Query_String);
    $this->Row = 0;
    odbc_binmode($this->QueryID, 1);
    odbc_longreadlen($this->QueryID, 4096);
    
		$this->Errno = odbc_error($this->LinkID); 
		$this->Error = odbc_errormsg($this->LinkID);
    if (!$this->QueryID) {
      $this->halt("Invalid SQL: ".$Query_String);
    }
		if ($this->RecordsPerPage && $this->PageNumber) {
			$ShiftRecords = (($this->PageNumber - 1) * $this->RecordsPerPage);
			$RecordsPerPage = $this->RecordsPerPage;
			while($ShiftRecords != 0) {
				$ShiftRecords--;
				$this->next_record();
			}
			$this->RecordsShown = $RecordsPerPage;
			$this->RecordsPerPage = 0;
			$this->PageNumber = 0;
		}
    return $this->QueryID;
  }
  
  function next_record() {
		if ($this->RecordsShown != -1) {
			if($this->RecordsShown) {
				$this->RecordsShown--;
			} else {
				$this->RecordsShown = -1;
				return false;
			}
		} 
    $this->Record = array();
    $stat = odbc_fetch_row($this->QueryID);
    if (!$stat) {
      if ($this->AutoFree)
      {
        odbc_free_result($this->QueryID);
        $this->QueryID = 0;
      };
    } else {
      $this->Row++;
      $count = odbc_num_fields($this->QueryID);
      for ($i=1; $i<=$count; $i++)
      {
        $field_value = odbc_result($this->QueryID, $i );
        $fieldname = ($this->Uppercase) ? strtoupper(odbc_field_name($this->QueryID, $i)) : odbc_field_name($this->QueryID, $i);
        $this->Record[$fieldname] = $field_value;
        $this->Record[$i - 1] = $field_value;
      }
    }
    return $stat;
  }

  function seek($pos) {
    $i = 0;
    while($i < $pos && @odbc_fetch_row($this->QueryID)) $i++;
    $this->Row += $i;
  }

  function affected_rows() {
    return odbc_num_rows($this->QueryID);
  }
  
  function num_rows() {
    $num_rows = odbc_num_rows($this->QueryID);
    return $num_rows;
  }
  
  function num_fields() {
    return count($this->Record)/2;
  }

  function f($Name, $field_type = TEXT) 
	{
    if($this->Uppercase) $Name = strtoupper($Name);
    if(isset($this->Record[$Name]))
		{
			$value = $this->Record[$Name];
			switch($field_type)
			{
				case DATETIME:
					$value = parse_date($this->DatetimeMask, $value);
					break;
				case DATE:
					$value = parse_date($this->DateMask, $value);
					break;
				case TIME:
					$value = parse_date($this->TimeMask, $value);
					break;
			}
      return $value; 
		}
    else 
      return "";
  }
  
  function free_result() {
    @odbc_free_result($this->QueryID);
    $this->QueryID = 0;
  }

  function close()
  {
    if ($this->QueryID) {
      $this->free_result();
    }
    if ($this->LinkID != 0) {
      odbc_close($this->LinkID);
      $this->LinkID = 0;
    }
  }  

  function close_all()
  {
    odbc_close_all ();
  }  
  
  function halt($message) {

    if ($this->HaltOnError == "no")
      return;

		print "<b>Database error:</b> " . $message . "<br>\n";
		print "<b>ODBC Error Code:</b> " . $this->Errno . "<br>\n";
		print "<b>ODBC Error Message:</b> " . $this->Error . "<br>\n";

    if ($this->HaltOnError != "report")
      die("Session halted.");
  }

	function tosql($value, $value_type, $is_delimiters = true, $use_null = true)
	{
    if(is_array($value) || strlen($value)) 
		{
			switch($value_type)
			{
				case NUMBER:
				case FLOAT:
	        return doubleval(str_replace(",", ".", $value));
					break;
				case TEXT:
					$value = str_replace("'", "''", $value);
					break;
				case DATETIME:
					if(!is_array($value) && is_int($value)) { $value = va_time($value); }
					if(is_array($value)) { $value = va_date($this->DatetimeMask, $value); }
					else { return "NULL"; }
					break;
				case INTEGER:
	        return intval($value);
					break;
				case DATE:
					if(!is_array($value) && is_int($value)) { $value = va_time($value); }
					if(is_array($value)) { $value = va_date($this->DateMask, $value); }
					else { return "NULL"; }
					break;
				case TIME:
					if(!is_array($value) && is_int($value)) { $value = va_time($value); }
					if(is_array($value)) { $value = va_date($this->TimeMask, $value); }
					else { return "NULL"; }
					break;
				case TIMESTAMP:
					if(!is_array($value) && is_int($value)) { $value = va_time($value); }
					if(is_array($value)) { $value = va_date($this->TimestampMask, $value); }
					else { return "NULL"; }
					break;
			}
			if($is_delimiters) {
				if($this->DBType == "access" && ($value_type == DATETIME || $value_type == DATE
					|| $value_type == TIME)) {
					$value = "#" . $value . "#";
				} else {
					$value = "'" . $value . "'";
				}
			}
		} 
		else if($use_null) 
		{
			$value = "NULL";
		} 
		else 
		{
			if($value_type == INTEGER || $value_type == FLOAT || $value_type == NUMBER) {
				$value = 0;
			} else if ($is_delimiters) {
				$value = "''";
			}
		} 
		return $value;
	}

}

?>